package features;

/**
 * Created by IntelliJ IDEA.
 * User: mac
 * Date: 25.06.2009
 * Time: 12:02:41
 * To change this template use File | Settings | File Templates.
 */

import java.util.*;
import java.io.BufferedReader;
import java.io.IOException;

import jm.music.data.Score;
import jm.music.data.Part;
import jm.music.data.Phrase;
import jm.music.data.Note;
import comparison.ACCEuclidean;
import comparison.ACComparison;


public class ACFDuration extends ACFeature
{
    private ArrayList<double[]> durations;
    
    public ACFDuration()
    {
        this.featureID = "Note Duration";
    }

    public String[] allowedCompareAlgorithms()
    {
        return ACComparison.getAllowedAlgorihtms(ACComparison.TYPE_DOUBLEVECT);
    }

     public String serialise()
    {
        String ser = featureID + "\nLines:" + durations.size() + "\n";
        for (double[] dur : durations)
        {
            for (double d : dur)
            {
                ser += d + ";";    
            }
            ser += "\n";
        }
        return ser;
    }

    public void deserialise(BufferedReader in)
    {
        try
        {
            //line 1:<numlines>

            String lone = in.readLine();

            lone = lone.substring(lone.indexOf(':')+1);

            int nlines = Integer.parseInt(lone);

            durations = new ArrayList<double[]>();

            for (int i = 0; i < nlines; i++)
            {
                String[] durs = in.readLine().split(";");

                double[] dline = new double[durs.length];

                for (int j = 0; j < durs.length; j++)
                {
                      dline[j] = Double.parseDouble(durs[j]);
                }

                durations.add(dline);
            }

        }
        catch (IOException ex)
        {
            //fail!
        }
    }

    public void print()
    {
        for (double[] dur : durations)
        {
            System.out.print("[duration]\t");
           for (double d : dur)
               System.out.print(d + ";");
            System.out.println("");
        }
    }

    public void extract (Score s)
    {
        durations = new ArrayList<double[]>();
        
        Enumeration e = s.getPartList().elements();

        while (e.hasMoreElements())
        {
           Vector<Double> line = new Vector<Double>();
            
           Part nextPt = (Part) e.nextElement();

           //filter out percussion track
           if (nextPt.getChannel() == 10) continue;

           String b = "";

           Vector<Note> notes = mergePhrases(nextPt);

           for (Note note : notes)
           {
              b += upOrDown(note.getPitch());
              line.add(note.getDuration());
           }

            /*
            Enumeration e2 = nextPt.getPhraseList().elements();

            while (e2.hasMoreElements())
            {
                Phrase nextPhr = (Phrase) e2.nextElement();

                Enumeration e3 = nextPhr.getNoteList().elements();

                while (e3.hasMoreElements())
                {
                    Note nextNote = (Note) e3.nextElement();
                    if (!nextNote.isRest())
                    {
                        b += upOrDown(nextNote.getPitch());

                        line.add(nextNote.getDuration());

                    }
                }
            }
            */

            if (!isBeat(b))
            {
                double[] dur = new double[line.size()];
                for (int i = 0; i < line.size(); i++)
                    dur[i] = (Double)line.get(i);

                durations.add(dur);
            }
        }
    }

    public double compare(ACFeature p)
    {
        //System.out.println("compare euclid!");
        //not a valid comparison
        if (ACFDuration.class != p.getClass()) return 0;

        ACFDuration cp = (ACFDuration)p;

        //compare each string to each string, return best match

        double top = 0;

        for (int i = 0; i < durations.size(); i++)
        {
            for   (int j = 0; j < cp.durations.size(); j++)
            {
                ACComparison cc = ACComparison.comparisonFactory(compareAlgorithm);
                double val = cc.compareDoublevect(durations.get(i), cp.durations.get(j));

                if (val > top) top = val;
            }
        }

        return top;
    }
}
