package features;

import jm.music.data.Score;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import comparison.ACCHamming;
import comparison.ACCLevenshtein;
import comparison.ACComparison;

/**
 * created by as part of features
 * User: mac
 * Date: 15.08.2009
 * Time: 17:21:45
 */
public class ACFLempelZiv extends ACFeature
{
    public Hashtable<String,Integer> fSet;

    public ACFLempelZiv()
    {
      this.featureID = "LempelZivWelch";
      fSet = new Hashtable<String, Integer>();
    }

    public String[] allowedCompareAlgorithms()
    {
        return ACComparison.getAllowedAlgorihtms(ACComparison.TYPE_STRING);
    }


    public void print()
    {
        for (String s : fSet.keySet())
		{
			System.out.println("[lzw]\t " + fSet.get(s) + ":" + s);
		}
    }

    public String serialise()
    {
        String ser = featureID + "\nLines:" + fSet.keySet().size() + "\n";
        for (String key : fSet.keySet())
        {
            ser += fSet.get(key) + ":" + key  + "\n";
        }
        return ser;
    }

    public void deserialise(BufferedReader in)
    {
        try
        {
            //line 1: <name/debug>:<numlines>

            String lone = in.readLine();

            lone = lone.substring(lone.indexOf(':')+1);

            int nlines = Integer.parseInt(lone);

            fSet = new Hashtable<String, Integer>();

            for (int i = 0; i < nlines; i++)
            {
                String[] info = in.readLine().split(":");

                fSet.put(info[1], Integer.parseInt(info[0]));
            }

        }
        catch (IOException ex)
        {
            System.out.println("failed to load lzw");
        }
    }

    public void extract(Score s)
    {
        Hashtable<Integer, String> dict = new Hashtable<Integer, String>();

        //extract from parsons code
        ACFParson acfp = new ACFParson();
        acfp.extract(s);

        //append all parsons codes to input string

        String input = "";
        String oinput = "";

        for (String code : acfp.pcodes)
        {
          input += code;
        }

        oinput = input;


        //lzw

        //initialise dictionary with 0,1,2 (all possible values)

        dict.put(0, "0");
        dict.put(1, "1");
        dict.put(2, "2");

        int count = 3;

        String entry = "";

        while (input.length() > 0)
        {
            String newchar = input.substring(0,1);    //get first character
            entry += newchar;
            input = input.substring(1);  //cut off first character

            if (dict.containsValue(entry)) continue;

            dict.put(count++, entry);

            entry = newchar; 
        }


        //dictionary complete, get 5 longest entries

        Hashtable<Integer, ArrayList<String>> sorts = new Hashtable<Integer, ArrayList<String>>();

        ArrayList<Integer> lengths = new ArrayList<Integer>();

        for (Integer i : dict.keySet())
        {
            String val = dict.get(i);

            if (sorts.containsKey(val.length()))
            {
                sorts.get(val.length()).add(val);
            }
            else
            {
                ArrayList<String> nl = new ArrayList<String>();
                nl.add(val);
                sorts.put(val.length(), nl);
                lengths.add(val.length());
            }

        }

        Collections.sort(lengths);
        Collections.reverse(lengths);


        //fill feature set variable

        for (Integer i : lengths)
        {
            for (String f : sorts.get(i))
            {
                //count occurance in input string and add to features

                int scount = countSubstring(f,oinput);

                if (scount > 1) // add only if string occurs more than once
                {
                    fSet.put(f, scount);
                }
            }

            if (fSet.values().size() >= 5) break;
        }

        //done!
    }

    public static int countSubstring(String needle, String haystack)
    {
        Matcher m = Pattern.compile(needle).matcher(haystack);

        int count =0;

        while (m.find()) count++;

        return count;
    }

    private static void debugDict(Hashtable<Integer, String> dict)
	{
		for (Integer i : dict.keySet())
		{
			System.out.println(i + ": " + dict.get(i));
		}
	}

    public double compare(ACFeature acf)
    {
        //not a valid comparison
        if (ACFLempelZiv.class != acf.getClass()) return 0;


        ACFLempelZiv cp = (ACFLempelZiv)acf;

        //compare each string to each string, return best match

        double top = 0;

        for (String key : fSet.keySet())
        {
            for (String fkey : cp.fSet.keySet())
            {
                ACComparison cc = ACComparison.comparisonFactory(compareAlgorithm);
                double val = cc.compareString(key, fkey);

                if (val > top) top = val;
            }
        }

        return top;
    }
}
