package features;

import jm.music.data.Score;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import comparison.ACCHamming;
import comparison.ACCLevenshtein;
import comparison.ACComparison;

/**
 * created by as part of features
 * User: mac
 * Date: 15.08.2009
 * Time: 17:21:45
 */
public class ACFLempelZivFrequency extends ACFeature
{
    public Hashtable<String,Integer> fSet;

    public ACFLempelZivFrequency()
    {
      this.featureID = "LempelZivWelchFrequency";
      fSet = new Hashtable<String, Integer>();
    }


        public String[] allowedCompareAlgorithms()
    {
        return ACComparison.getAllowedAlgorihtms(ACComparison.TYPE_STRING);
    }

    public void print()
    {
        for (String s : fSet.keySet())
		{
			System.out.println("[lzwF]\t " + fSet.get(s) + ":" + s);
		}
    }

    public String serialise()
    {
        String ser = featureID + "\nLines:" + fSet.keySet().size() + "\n";
        for (String key : fSet.keySet())
        {
            ser += fSet.get(key) + ":" + key  + "\n";
        }
        return ser;
    }

    public void deserialise(BufferedReader in)
    {
        try
        {
            //line 1: <name/debug>:<numlines>

            String lone = in.readLine();

            lone = lone.substring(lone.indexOf(':')+1);

            int nlines = Integer.parseInt(lone);

            fSet = new Hashtable<String, Integer>();

            for (int i = 0; i < nlines; i++)
            {
                String[] info = in.readLine().split(":");

                fSet.put(info[1], Integer.parseInt(info[0]));
            }

        }
        catch (IOException ex)
        {
            System.out.println("failed to load lzwF");
        }
    }

    public void extract(Score s)
    {
        Hashtable<Integer, String> dict = new Hashtable<Integer, String>();

        //extract from parsons code
        ACFParson acfp = new ACFParson();
        acfp.extract(s);

        //append all parsons codes to input string

        String input = "";
        String oinput = "";

        for (String code : acfp.pcodes)
        {
          input += code;
        }

        oinput = input;


        //lzw

        //initialise dictionary with 0,1,2 (all possible values)

        dict.put(0, "0");
        dict.put(1, "1");
        dict.put(2, "2");

        int count = 3;

        String entry = "";

        while (input.length() > 0)
        {
            String newchar = input.substring(0,1);    //get first character
            entry += newchar;
            input = input.substring(1);  //cut off first character

            if (dict.containsValue(entry)) continue;

            dict.put(count++, entry);

            entry = newchar;
        }


        //get 5 most frequent n-grams with min. length of 4



        //dictionary complete, get 5 longest entries

        Hashtable<Integer, ArrayList<String>> sorts = new Hashtable<Integer, ArrayList<String>>();

        ArrayList<Integer> lengths = new ArrayList<Integer>();

        for (Integer i : dict.keySet())
        {
            String val = dict.get(i);

            int numocc = countSubstring(val,oinput);

            if (sorts.containsKey(numocc))
            {
                sorts.get(numocc).add(val);
            }
            else
            {
                ArrayList<String> nl = new ArrayList<String>();
                nl.add(val);
                sorts.put(numocc, nl);
                lengths.add(numocc);
            }

        }

        Collections.sort(lengths);
        Collections.reverse(lengths);


        //fill feature set variable

        for (Integer i : lengths)
        {
            for (String f : sorts.get(i))
            {
                //count occurance in input string and add to features

                if (f.length() >= 4) // add only if length >= 4
                {
                    fSet.put(f, i);
                }
            }

            if (fSet.values().size() >= 10) break;
        }

        //done!
    }

    public static int countSubstring(String needle, String haystack)
    {
        Matcher m = Pattern.compile(needle).matcher(haystack);

        int count =0;

        while (m.find()) count++;

        return count;
    }

    private static void debugDict(Hashtable<Integer, String> dict)
	{
		for (Integer i : dict.keySet())
		{
			System.out.println(i + ": " + dict.get(i));
		}
	}

    public double compare(ACFeature acf)
    {
        //not a valid comparison
        if (ACFLempelZivFrequency.class != acf.getClass()) return 0;


        ACFLempelZivFrequency cp = (ACFLempelZivFrequency)acf;

        //compare each string to each string, return best match

        double top = 0;

        double count = 0;
        double vals = 0;
        
        for (String key : fSet.keySet())
        {
            for (String fkey : cp.fSet.keySet())
            {
                ACComparison cc = ACComparison.comparisonFactory(compareAlgorithm);
                double val = cc.compareString(key, fkey);

                //if (val > top) top = val;
                vals += val;
                //System.out.println("v=" + val);
                count++;
            }
        }
        //System.out.println("vals=" +vals + "\tcount=" + count);
        return (vals/count);
    }
}
