package features;

import jm.music.data.Score;
import jm.music.data.Part;
import jm.music.data.Note;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;

import comparison.ACComparison;

/**
 * created by as part of features
 * User: mac
 * Date: 14.08.2009
 * Time: 13:13:22
 */
public class ACFNoteHistogram extends ACFeature
{
    Hashtable<String, Integer> histogram;

    public ACFNoteHistogram()
    {
        this.featureID = "Note Histogram";
        histogram = new Hashtable<String, Integer>();
    }

    public double[] getCompareVector()
    {
        double[] vect = new double[14];

        vect[0] = getCompValFor(Note.A);
        vect[1] = getCompValFor(Note.A_FLAT);
        vect[2] = getCompValFor(Note.A_SHARP);

        vect[3] = getCompValFor(Note.B);
        vect[4] = getCompValFor(Note.B_FLAT);

        vect[5] = getCompValFor(Note.C);
        vect[6] = getCompValFor(Note.C_SHARP);

        vect[7] = getCompValFor(Note.D);
        vect[8] = getCompValFor(Note.D_SHARP);

        vect[9] = getCompValFor(Note.E);
        vect[10] = getCompValFor(Note.E_FLAT);

        vect[10] = getCompValFor(Note.F);
        vect[11] = getCompValFor(Note.F_SHARP);

        vect[12] = getCompValFor(Note.G);
        vect[13] = getCompValFor(Note.G_SHARP);
        

        return vect;
    }

    public double getCompValFor(String note)
    {
        if (histogram.containsKey(note)) return (double)histogram.get(note);
        return 0d;
    }

    public String[] allowedCompareAlgorithms()
    {
        return ACComparison.getAllowedAlgorihtms(ACComparison.TYPE_DOUBLEVECT);
    }

    public void print()
    {
        for (String s : histogram.keySet())
		{
			System.out.println("[note histogram]\t " + s + ":" + histogram.get(s));
		}
    }

    public String serialise()
    {
        String ser = featureID + "\nLines:" + histogram.keySet().size() + "\n";
        for (String key : histogram.keySet())
        {
            ser += key + ":" + histogram.get(key) + "\n";
        }
        return ser;
    }

    public void deserialise(BufferedReader in)
    {
        try
        {
            //line 1: <name/debug>:<numlines>

            String lone = in.readLine();

            lone = lone.substring(lone.indexOf(':')+1);

            int nlines = Integer.parseInt(lone);

            histogram = new Hashtable<String, Integer>();

            for (int i = 0; i < nlines; i++)
            {
                String[] noteinfo = in.readLine().split(":");

                histogram.put(noteinfo[0], Integer.parseInt(noteinfo[1]));
            }

        }
        catch (IOException ex)
        {
            System.out.println("failed to load note histogram");
        }
    }

    public void extract(Score s)
    {
        //TODO:

        histogram = new Hashtable<String, Integer>();

        Enumeration e = s.getPartList().elements();

        while (e.hasMoreElements())
        {
            Part nextPt = (Part) e.nextElement();

            //filter out percussion track
            if (nextPt.getChannel() == 10) continue;

            String b = "";

            Vector<Note> notes = mergePhrases(nextPt);

            for (Note note : notes)
            {
               if (!note.isRest())
               {
                   String n = note.getNote();
                   
                   if (histogram.containsKey(n))
                   {
                       Integer c = (Integer)histogram.get(n);

                       histogram.put(n, c+1);
                   }
                   else
                   {
                       histogram.put(n, 1);
                   }
               }
            }
        }
    }


    public double compare(ACFeature acc)
    {
        if (ACFNoteHistogram.class != acc.getClass()) return 0;

        ACFNoteHistogram cp = (ACFNoteHistogram)acc;

        for (double d : getCompareVector())
        {
            System.out.print(d + " ");

        }
        /*
        System.out.println("");
        for (String nt : histogram.keySet())
            System.out.print(nt + " ");

        System.out.println("contains [" +Note.A_SHARP+ "]= " + histogram.keySet().contains(Note.A_SHARP));
          */
        ACComparison cc = ACComparison.comparisonFactory(compareAlgorithm);
        double val = cc.compareDoublevect(getCompareVector(), cp.getCompareVector());

        return val;
    }
}
