package features;

/**
 * Created by IntelliJ IDEA.
 * User: mac
 * Date: 25.06.2009
 * Time: 11:24:28
 * To change this template use File | Settings | File Templates.
 */

import java.util.*;
import java.io.BufferedReader;
import java.io.IOException;

import jm.music.data.Score;
import jm.music.data.Part;
import jm.music.data.Phrase;
import jm.music.data.Note;
import comparison.ACCDice;
import comparison.ACCLevenshtein;
import comparison.ACComparison;

public class ACFNoteSequence extends ACFeature
{
    public ArrayList<ArrayList<String>> seqs;

    public ACFNoteSequence()
    {
        this.featureID = "Note Sequence";
    }

    public String[] allowedCompareAlgorithms()
    {
        return ACComparison.getAllowedAlgorihtms(ACComparison.TYPE_STRING);
    }

    public String serialise()
    {
        String ser = featureID + "\nLines:" + seqs.size() + "\n";
        for (ArrayList<String> seq : seqs)
        {
            for (String s : seq)
                ser += s + "\t";

            ser += "\n";
        }
        return ser;
    }

    public void deserialise(BufferedReader in)
    {
        try
        {
            //line 1: <name/debug>:<numlines>

            String lone = in.readLine();

            lone = lone.substring(lone.indexOf(':')+1);

            int nlines = Integer.parseInt(lone);

            seqs = new ArrayList<ArrayList<String>>();


            for (int i = 0; i < nlines; i++)
            {
                ArrayList<String> seq = new ArrayList<String>();
                String[] lne = in.readLine().split("\t");

                for (String l : lne)
                    seq.add(l);

                seqs.add(seq);
            }

        }
        catch (IOException ex)
        {
            //fail!
        }
    }

    public void print()
    {

        for (ArrayList<String> seq : seqs)
        {
            System.out.print("[note seqence]\t");
            for (String s : seq)
                System.out.print(s + " ");

            System.out.println("");
        }
    }

    public void extract (Score s)
    {
        seqs = new ArrayList<ArrayList<String>>();

        Enumeration e = s.getPartList().elements();

        while (e.hasMoreElements())
        {
            Part nextPt = (Part) e.nextElement();

            //filter out percussion track
            if (nextPt.getChannel() == 10) continue;

            ArrayList<String> seq = new ArrayList<String>(); 

            Vector<Note> notes = mergePhrases(nextPt);

            for (Note note : notes)
            {
               if (!note.isRest())
               {
                   String n = note.getNote();

                  seq.add(n);
               }
            }

            seqs.add(seq);
        }

        //System.out.println("[note sequence]\textracted " + seqs.size() + " lines");
    }

    public double compare(ACFeature p)
    {
         //not a valid comparison
        if (ACFNoteSequence.class != p.getClass()) return 0;


        ACFNoteSequence cp = (ACFNoteSequence)p;

        //compare each string to each string, return best match

        double top = 0;

        for (int i = 0; i < seqs.size(); i++)
        {
            String[] x = new String[seqs.get(i).size()];
            int k = 0;
            for (String s : seqs.get(i))
                x[k++] = s;

            for   (int j = 0; j < cp.seqs.size(); j++)
            {
                String[] y = new String[cp.seqs.get(j).size()];
                int l = 0;
                for (String s : cp.seqs.get(j))
                    y[l++] = s;

                ACComparison cc = ACComparison.comparisonFactory(compareAlgorithm);
                double val = cc.compareStringvect(x,y);

                if (val > top) top = val;
            }
        }

        return top;
    }




}
