package features;

import jm.music.data.Score;
import jm.music.data.Part;
import jm.music.data.Note;
import jm.music.data.Phrase;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

import comparison.ACComparison;

/**
 * Created by IntelliJ IDEA.
 * User: mac
 * Date: 15.10.2009
 * Time: 15:44:30
 * To change this template use File | Settings | File Templates.
 */
public class ACFStatistics extends ACFeature
{
    public static int VEC_SIZE = 22;

    public double[] stats;

    private int dup;
    private int ddn;
    private int dsm;

    private int pptch;

    private int highNote;
    private int lowNote;

    
    public ACFStatistics()
    {
        this.featureID = "Statistics";
    }

    public static String[] getDescriptions()
    {
        String[] l = new String[VEC_SIZE];

        l[0] 	= "TimeSignatureDenominator";
        l[1] 	= "TimeSignatureNumerator";
        l[2] 	= "HighestPitch";
        l[3] 	= "LowestPitch";
        l[4] 	= "KeyQuality(minor/major)";
        l[5] 	= "KeySignature";
        l[6] 	= "LongestRhythmValue";
        l[7] 	= "ShortestRhythmValue";
        l[8] 	= "Tempo";
        l[9] 	= "Volume";
        l[10] = "changes/same";
        l[11] = "changes/up";
        l[12] = "changes/down";

        l[13] = "avg pitch";
        l[14] = "avg rhythm";

        //originally: highest/lowest/average frequency
        //now: pitch variance/skewness/kurtosis

        l[15] = "pitch variance";
        l[16] = "pitch skewness";
        l[17] = "pitch kurtosis";



        l[18] = "score size";

        l[19] = "rests";
        l[20] = "flats";
        l[21] = "sharps";


        return l;
    }
    
    public void extract(Score s)
    {
        stats = new double[VEC_SIZE];

        stats[0] = s.getDenominator();
        stats[1] = s.getNumerator();
        stats[2] = s.getHighestPitch();
        stats[3] = s.getLowestPitch();
        stats[4] = s.getKeyQuality();
        stats[5] = s.getKeySignature();
        stats[6] = s.getLongestRhythmValue();
        stats[7] = s.getShortestRhythmValue();
        stats[8] = s.getTempo();
        stats[9] = s.getVolume();



        dup = dsm = ddn = 0;
        pptch = 60;

        double avgp = 0;
        double avgr = 0;

        double hfreq = 0;
        double lfreq = 0;
        double afreq = 0;

        double cnt = 0;

        double rests = 0;
        double flats = 0;
        double sharps = 0;

        Enumeration e = s.getPartList().elements();

        while(e.hasMoreElements())
        {
        	Part nextPt = (Part)e.nextElement();
        	Enumeration enum2 = nextPt.getPhraseList().elements();
            while(enum2.hasMoreElements())
            {
                Phrase nextPhr = (Phrase)enum2.nextElement();
                Enumeration enum3 = nextPhr.getNoteList().elements();
                while(enum3.hasMoreElements())
                {
                    Note nextNote = (Note)enum3.nextElement();

                    if (nextNote.isRest()) rests++;
                    if (nextNote.isFlat()) flats++;
                    if (nextNote.isSharp()) sharps++;

                    int pitch = nextNote.getPitch();
                    double freq = nextNote.getFrequency();

                    cnt++;
                    avgp += pitch;
                    avgr += nextNote.getRhythmValue();

                    afreq += freq;
                    if (freq > hfreq) hfreq = freq;
                    if (freq < lfreq) lfreq = freq;


					changes(pitch);
            	}
        	}
    	}

        stats[10] = dsm;
        stats[11] = dup;
        stats[12] = ddn;

        stats[13] = avgp/cnt;
        stats[14] = avgr/cnt;

        //stats[15] = hfreq;
        //stats[16] = lfreq;
        //stats[17] = afreq/cnt;

        //originally: highest/lowest/average frequency
        //now: pitch variance/skewness/kurtosis

        double meanpitch = avgp/cnt;

        double variance = 0;
        double m3 = 0;
        double m4 = 0;

        cnt = 0;

        //2nd pass
        e = s.getPartList().elements();
        while(e.hasMoreElements())
        {
        	Part nextPt = (Part)e.nextElement();
        	Enumeration enum2 = nextPt.getPhraseList().elements();
            while(enum2.hasMoreElements())
            {
                Phrase nextPhr = (Phrase)enum2.nextElement();
                Enumeration enum3 = nextPhr.getNoteList().elements();
                while(enum3.hasMoreElements())
                {
                    Note nextNote = (Note)enum3.nextElement();

                    double pitch = (double)nextNote.getPitch();

                    variance +=  Math.pow(pitch - meanpitch, 2);
                    m3 +=  Math.pow(pitch - meanpitch, 3);
                    m4 +=  Math.pow(pitch - meanpitch, 4);
                    cnt++;
             	}
        	}
    	}

        System.out.println("variance=" + variance + " count=" + cnt);
        m3 = m3/cnt;
        m4 = m4/cnt;
        variance = variance/cnt;

        //variance
        stats[15] = variance;
        //skewness
        stats[16] = m3 / Math.pow(Math.sqrt(variance), 3);
        //kurtosis
        stats[17] = m4 / Math.pow(variance, 2);

        //if (stats[15] == Double.NaN) stats[15] = 0;
        //if (stats[16] == Double.NaN) stats[16] = 0;
        //if (stats[16] == Double.NaN) stats[16] = 0;

        stats[18] = s.getSize();

        stats[19] = rests;
        stats[20] = flats;
        stats[21] = sharps;

        
    }

    private void changes(int pitch)
    {
        if(pitch < pptch) ddn++;
	    if(pitch > pptch) dup++;
        if (pitch == pptch) dsm++;
	    pptch = pitch;
    }


    public double compare(ACFeature f)
    {
        System.out.println("compare statistics!");

        if (ACFStatistics.class != f.getClass()) return 0;

        ACFStatistics cp = (ACFStatistics)f;

        ACComparison cc = ACComparison.comparisonFactory(compareAlgorithm);
        double val = cc.compareDoublevect(stats, cp.stats);

        System.out.println("compare A");
        for (double d : stats)
            System.out.print(d + ";");
        System.out.println("");
        System.out.println("compare B");
        for (double d : cp.stats)
            System.out.print(d + ";");
        System.out.println("");

        return val;
    }

    public void print()
    {
        System.out.print("[statistics]\t");
        for (double s : stats)
        {
            System.out.print(s + ";");
        }
        System.out.println("");
    }

    public String serialise()
    {
        String ser = featureID + "\nLines:" + "1" + "\n";
        for (double s : stats)
        {
           ser += s + ";";        
        }
        ser += "\n";
        return ser;
    }

    public void deserialise(BufferedReader in)
    {
    try
        {
            //line 1:<numlines>

            String lone = in.readLine();

            String[] durs = in.readLine().split(";");

            stats = new double[durs.length];

            for (int j = 0; j < durs.length; j++)
            {
                  stats[j] = Double.parseDouble(durs[j]);
            }
        }
        catch (IOException ex)
        {
            //fail!
        }
    }

    public String[] allowedCompareAlgorithms()
    {
         return ACComparison.getAllowedAlgorihtms(ACComparison.TYPE_DOUBLEVECT);
    }
}
