package features; /**
 * Created by IntelliJ IDEA.
 * User: mac
 * Date: 25.06.2009
 * Time: 11:18:19
 * To change this template use File | Settings | File Templates.
 */

import jm.music.data.Score;
import jm.music.data.*;

import java.util.*;
import java.io.BufferedReader;

public abstract class ACFeature implements java.io.Serializable
{
    private int prev = 60;

    public static double    DEFAULT_BEAT_THRESHOLD = 0.2;
    public static char      DOWN	= '0';
    public static char      UP 	= '1';
    public static char      SAME = '2';

    
    public String featureID = "unnamed feature";
    
    public abstract void extract(Score s);

    public abstract double compare(ACFeature f);

    public abstract void print();

    public abstract String serialise();

    public abstract void deserialise(BufferedReader in);

    public String compareAlgorithm = "";

    public abstract String[] allowedCompareAlgorithms();


    public static String[] getFeatureIds()
    {
        String[] res = new String[7];

        res[0] = "Note Duration";
        res[1] = "LempelZivWelch";
        res[2] = "LempelZivWelchFrequency";
        res[3] = "Note Histogram";
        res[4] = "Parsons Code";
        res[5] = "Note Sequence";
        res[6] = "Statistics";

        return res;
    }

     public static ACFeature featureFactory(String id)
    {
       if (id.equals("Note Duration")) return new ACFDuration();
       else if (id.equals("LempelZivWelch")) return new ACFLempelZiv();
        else if (id.equals("LempelZivWelchFrequency")) return new ACFLempelZivFrequency();
        else if (id.equals("Note Histogram")) return new ACFNoteHistogram();
        else if (id.equals("Parsons Code")) return new ACFParson();
        else if (id.equals("Note Sequence")) return new ACFNoteSequence();
        else if (id.equals("Statistics")) return new ACFStatistics();


       return null;
    }
    /*
     helper functions
     */

    public byte upOrDown(int pitch)
	{
        byte b = 0;
        if (pitch > prev) b = 1;
        if (pitch == prev) b = 2;
        prev = pitch;
        return b;
    }

	public boolean isBeat(String track)
    {
  	    return isBeat(track, DEFAULT_BEAT_THRESHOLD);
    }

    public boolean isBeat(String track, double threshold)
    {
        double tlen = track.length();
        double changes = 0;

        for (int i = 0; i < tlen; i++)
        {
            if (track.charAt(i) != SAME) changes++;
        }

        double val = changes / tlen;

        return (val <= threshold);
    }

    public Vector<Note> mergePhrases(Part pt)
    {                                           
        Vector<Note> notelist = new Vector<Note>();

        Hashtable<Double,Note> notemap = new Hashtable<Double,Note>();

        Enumeration e2 = pt.getPhraseList().elements();

        while (e2.hasMoreElements())
        {
            Phrase nextPhr = (Phrase) e2.nextElement();
            Enumeration e3 = nextPhr.getNoteList().elements();

            int nc = 0;   //note counter

            while (e3.hasMoreElements())
            {
                Note nextNote = (Note) e3.nextElement();

                double st = nextPhr.getNoteStartTime(nc);

                if (notemap.containsKey(st))
                {
                    Note old = (Note)notemap.get(st);

                    if (old.getPitch() < nextNote.getPitch())
                        notemap.put(st, nextNote);
                }
                else
                {
                    notemap.put(st, nextNote);
                }

                nc++;
            }
        }

        Enumeration keys = notemap.keys();

        ArrayList<Double> keylist = new ArrayList<Double>();

        while (keys.hasMoreElements())
        {
            double k = (Double)keys.nextElement();

            keylist.add(k);
        }

        double[] klist = new double[keylist.size()];

        for (int i = 0; i < keylist.size(); i++)
            klist[i] = (Double)keylist.get(i);

        Arrays.sort(klist);

        for (int i = 0; i < klist.length; i++)
        {
            notelist.add(notemap.get(klist[i]));
        }




        return notelist;
    }
}
