package gui;

import main.*;

import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Hashtable;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.builder.DefaultFormBuilder;
import features.ACFeature;

import jm.music.data.*;
import jm.gui.cpn.PianoStave;
import jm.gui.cpn.Stave;
import jm.util.Play;
import jm.util.Read;
import SomLib.SomLibWriter;

/**
 * Created by IntelliJ IDEA.
 * User: Christian Seidl
 * Date: 15.09.2009
 * Time: 00:06:29
 * To change this template use File | Settings | File Templates.
 */
public class ACGuiMain extends Component {
    public static void main(String[] args) {
        JFrame frame = new JFrame("ACGuiMain");
        frame.setContentPane(new ACGuiMain().panel1);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }

    private JPanel panel1;
    private JTextField tfLibFolder;
    private JButton btnLoadLibFolder;
    private JButton btnAnalyseMidiFolder;
    private JTextField tfMidiFolder;
    private JTextArea taOutput;
    private JPanel midiPanel;
    private JTextField tfMidiFile;
    private JPanel featuresContainerPanel;
    private JButton btnAddFeature;
    private JButton btnGo;
    private JPanel stavePanel;
    private JButton btnClearStave;
    private JButton btnPlayStave;
    private JPanel tablePanel;
    private JButton btnSomlib;
    private JButton btnStopLP;
    private JButton btnPlayCompareFile;
    private JButton btnPlayTableSelection;

    private ACompare comp;
    private File libFolder;
    private File midiFolder;
    private File midiFile;
    private HashSet<String> compFiles;
    private JTable outputTable;

    private String[] features;
    private String[] comps;
    private double[] weights;

    private Phrase examplePhrase;

    public ACGuiMain() {
        comp = new ACompare();
        compFiles = new HashSet<String>();

        final JFileChooser fc = new JFileChooser(".");

        examplePhrase = new Phrase("examplePhrase");
        Stave ps = new PianoStave(examplePhrase);
        stavePanel.add(ps);

        addFeatureSelect();

        btnLoadLibFolder.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (libFolder != null) {
                    printTA("load meta!");
                    String[] files = comp.lib.loadMeta(libFolder.getAbsolutePath());
                    setMidiChkBoxes(files);
                } else {
                    printTA("please choose lib folder");
                }
            }
        });
        btnAnalyseMidiFolder.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                clearTA();
                taOutput.setText("analyse meta - please wait...");
                taOutput.paintImmediately(taOutput.getBounds());
                if (libFolder != null && midiFolder != null) {
                    comp.lib.analyseMeta(midiFolder.getAbsolutePath(), libFolder.getAbsolutePath());
                    printTA("Meta files stored in " + libFolder.getAbsolutePath());
                } else {
                    clearTA();
                    printTA("please choose lib and midi folder");
                }
            }
        });
        tfLibFolder.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent me) {
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                if (me.getClickCount() == 2) {
                    int returnVal = fc.showOpenDialog(ACGuiMain.this);

                    if (returnVal == JFileChooser.APPROVE_OPTION) {
                        libFolder = fc.getSelectedFile();
                        tfLibFolder.setText(libFolder.getAbsolutePath());
                    }
                }
            }
        });
        tfMidiFolder.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent me) {
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                if (me.getClickCount() == 2) {
                    int returnVal = fc.showOpenDialog(ACGuiMain.this);

                    if (returnVal == JFileChooser.APPROVE_OPTION) {
                        midiFolder = fc.getSelectedFile();
                        tfMidiFolder.setText(midiFolder.getAbsolutePath());
                    }
                }
            }
        });
        tfMidiFile.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent me) {
                fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                fc.setFileFilter(new MIDIFileFilter());
                if (me.getClickCount() == 2) {
                    int returnVal = fc.showOpenDialog(ACGuiMain.this);

                    if (returnVal == JFileChooser.APPROVE_OPTION) {
                        midiFile = fc.getSelectedFile();
                        tfMidiFile.setText(midiFile.getAbsolutePath());
                    }
                }
            }
        });

        btnAddFeature.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                addFeatureSelect();
            }
        });
        btnGo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Component featurePanels[] = featuresContainerPanel.getComponents();
                int len = featurePanels.length;
                features = new String[len];
                comps = new String[len];
                weights = new double[len];

                clearTA();

                printTA(compFiles.toString());

                int i = 0;
                for (Component fp : featurePanels) {
                    //System.out.println(fp.getClass());
                    JPanel p = (JPanel) fp;
                    Component fpcomps[] = p.getComponents();
                    JComboBox feat = (JComboBox) fpcomps[0];
                    JComboBox comp = (JComboBox) fpcomps[1];
                    JLabel sl = (JLabel) fpcomps[3];
                    //System.out.println(feat.getSelectedItem()+" "+comp.getSelectedItem()+" "+sl.getText());
                    features[i] = (String) feat.getSelectedItem();
                    comps[i] = (String) comp.getSelectedItem();
                    weights[i] = Double.parseDouble(sl.getText());
                    i++;
                }

                if (libFolder != null) {

                    ArrayList<ACMeta> metas = new ArrayList<ACMeta>();

                    if (compFiles.size() > 0) {
                        for (String cf : compFiles) {
                            for (ACMeta lm : comp.lib.mfiles) {
                                if (cf.equals(lm.relPath)) { //(lm.sourceFile.substring(lm.sourceFile.lastIndexOf("/") + 1) + ".meta").equals(cf)) {
                                    metas.add(lm);
                                }
                            }
                        }
                    } else {
                        metas = comp.lib.mfiles;
                    }
                    ACMeta in = new ACMeta();

                    if (midiFile != null) {
                        in.analyseFile(midiFile.getAbsolutePath());
                        createOutputTable(ACompare.compareAllAgainstList(in, metas, features, comps, weights));
                        printTA(ACompare.printResults(ACompare.compareAllAgainstList(in, metas, features, comps, weights)));

                    } else if (examplePhrase.getSize() != 0) {
                        in.analyseFile(new Score(new Part(examplePhrase)));
                        createOutputTable(ACompare.compareAllAgainstList(in, metas, features, comps, weights));
                        printTA(ACompare.printResults(ACompare.compareAllAgainstList(in, metas, features, comps, weights)));

                    } else {
                        printTA("please choose midi file or enter example");
                    }
                } else {
                    printTA("please choose lib folder");
                }
            }
        });
        btnClearStave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stavePanel.removeAll();
                examplePhrase = new Phrase("examplePhrase");
                Stave ps = new PianoStave(examplePhrase);
                stavePanel.add(ps);
                stavePanel.revalidate();
            }
        });
        btnPlayStave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Play.midi(examplePhrase, false);
            }
        });
        btnSomlib.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                clearTA();

                if (comp.lib.mfiles.size() == 0) {
                    printTA("please load lib");
                } else {
                    fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                    int returnVal = fc.showOpenDialog(ACGuiMain.this);

                    if (returnVal == JFileChooser.APPROVE_OPTION) {
                        File somlibfile = fc.getSelectedFile();
                        SomLibWriter.writeInputVectorFile(somlibfile.getAbsolutePath(), comp.lib);
                        SomLibWriter.writeTemplateVectorFile(somlibfile.getAbsolutePath(), comp.lib);
                        printTA("writing SomLib to " + somlibfile.getName());
                    }
                }
            }
        });
        btnStopLP.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Play.stopMidi();
            }
        });
        btnPlayCompareFile.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!tfMidiFile.getText().equals("")) {
                    printTA("playing " + tfMidiFile.getText());

                    Score ps = new Score();
                    Read.midi(ps, tfMidiFile.getText());
                    Play.midi(ps, false);
                } else {
                    printTA("Please choose MidiFile to Play");
                }
            }
        });
        btnPlayTableSelection.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int row = outputTable.getSelectedRow();

                for (ACMeta m : comp.lib.mfiles) {
                    if (m.relPath.equals(outputTable.getValueAt(row, 0))) {
                        File f = new File(m.sourceFile);
                        if (f.exists()) {
                            printTA("playing " + m.sourceFile);

                            Score ps = new Score();
                            Read.midi(ps, m.sourceFile);
                            Play.midi(ps, false);

                        } else {
                            printTA("could not find source file " + m.sourceFile);
                        }
                    }
                }

            }
        });
    }

    private void addFeatureSelect() {
        ACFeatureSelect fSelect = new ACFeatureSelect();
        JPanel fPanel = fSelect.panelFeatureSelect;
        fSelect.setFeatures(ACFeature.getFeatureIds());
        featuresContainerPanel.add(fPanel);
        featuresContainerPanel.setPreferredSize(new Dimension(featuresContainerPanel.getWidth(), featuresContainerPanel.getComponents().length * 35));
        featuresContainerPanel.revalidate();
    }

    {
// GUI initializer generated by IntelliJ IDEA GUI Designer
// >>> IMPORTANT!! <<<
// DO NOT EDIT OR ADD ANY CODE HERE!
        $$$setupUI$$$();
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$() {
        panel1 = new JPanel();
        panel1.setLayout(new FormLayout("fill:60px:noGrow,left:4dlu:noGrow,fill:max(d;200px):grow,left:4dlu:noGrow,fill:100px:noGrow,left:6dlu:noGrow,fill:350px:noGrow,left:4dlu:noGrow,fill:350px:noGrow,left:4dlu:noGrow,fill:max(d;4px):noGrow", "center:10px:noGrow,top:4dlu:noGrow,center:max(d;4px):noGrow,top:4dlu:noGrow,center:max(d;4px):noGrow,top:4dlu:noGrow,center:24px:noGrow,top:15dlu:noGrow,center:32px:noGrow,top:5dlu:noGrow,center:250px:noGrow,top:4dlu:noGrow,center:max(d;4px):noGrow,top:4dlu:noGrow,center:150px:noGrow,top:4dlu:noGrow,center:max(d;4px):noGrow"));
        tfLibFolder = new JTextField();
        tfLibFolder.setText("dbl-click for open");
        CellConstraints cc = new CellConstraints();
        panel1.add(tfLibFolder, cc.xy(3, 5, CellConstraints.FILL, CellConstraints.FILL));
        btnLoadLibFolder = new JButton();
        btnLoadLibFolder.setHorizontalAlignment(0);
        btnLoadLibFolder.setHorizontalTextPosition(11);
        btnLoadLibFolder.setText("load");
        btnLoadLibFolder.setToolTipText("load library folder");
        panel1.add(btnLoadLibFolder, cc.xy(5, 5, CellConstraints.FILL, CellConstraints.DEFAULT));
        tfMidiFolder = new JTextField();
        tfMidiFolder.setText("dbl-click for open");
        panel1.add(tfMidiFolder, cc.xy(3, 7, CellConstraints.FILL, CellConstraints.FILL));
        btnAnalyseMidiFolder = new JButton();
        btnAnalyseMidiFolder.setText("analyse");
        btnAnalyseMidiFolder.setToolTipText("analyse midi folder and store to library");
        panel1.add(btnAnalyseMidiFolder, cc.xy(5, 7));
        final JLabel label1 = new JLabel();
        label1.setText(" Lib folder:");
        panel1.add(label1, cc.xy(1, 5));
        final JLabel label2 = new JLabel();
        label2.setText(" Midi folder:");
        panel1.add(label2, cc.xy(1, 7));
        tfMidiFile = new JTextField();
        tfMidiFile.setText("dbl-click for open");
        panel1.add(tfMidiFile, cc.xy(3, 3, CellConstraints.FILL, CellConstraints.FILL));
        final JLabel label3 = new JLabel();
        label3.setText(" Midi file:");
        panel1.add(label3, cc.xy(1, 3));
        //final com.intellij.uiDesigner.core.Spacer spacer1 = new com.intellij.uiDesigner.core.Spacer();
        //panel1.add(spacer1, cc.xy(7, 1, CellConstraints.DEFAULT, CellConstraints.FILL));
        btnAddFeature = new JButton();
        btnAddFeature.setText("add Feature");
        panel1.add(btnAddFeature, cc.xy(7, 3));
        btnGo = new JButton();
        btnGo.setText("GO !!!");
        panel1.add(btnGo, cc.xy(9, 3));
        btnPlayStave = new JButton();
        btnPlayStave.setText("play");
        panel1.add(btnPlayStave, cc.xy(7, 17));
        btnClearStave = new JButton();
        btnClearStave.setText("clear");
        panel1.add(btnClearStave, cc.xy(9, 17));
        stavePanel = new JPanel();
        stavePanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
        stavePanel.setMinimumSize(new Dimension(200, 200));
        stavePanel.setPreferredSize(new Dimension(660, 180));
        panel1.add(stavePanel, cc.xyw(7, 15, 3, CellConstraints.DEFAULT, CellConstraints.FILL));
        final JScrollPane scrollPane1 = new JScrollPane();
        panel1.add(scrollPane1, cc.xyw(1, 15, 5, CellConstraints.FILL, CellConstraints.FILL));
        taOutput = new JTextArea();
        taOutput.setEditable(false);
        taOutput.setEnabled(true);
        taOutput.setLineWrap(false);
        taOutput.setRows(10);
        taOutput.setText("");
        scrollPane1.setViewportView(taOutput);
        tablePanel = new JPanel();
        tablePanel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
        tablePanel.setPreferredSize(new Dimension(10, 10));
        panel1.add(tablePanel, cc.xyw(7, 11, 3, CellConstraints.DEFAULT, CellConstraints.FILL));
        btnSomlib = new JButton();
        btnSomlib.setText("SomLib");
        panel1.add(btnSomlib, cc.xyw(1, 17, 5));
        btnStopLP = new JButton();
        btnStopLP.setFont(new Font(btnStopLP.getFont().getName(), Font.BOLD, btnStopLP.getFont().getSize()));
        btnStopLP.setForeground(new Color(-16777216));
        btnStopLP.setText("stop midi playback");
        btnStopLP.setToolTipText("analyse midi folder and store to library");
        panel1.add(btnStopLP, cc.xyw(1, 9, 5, CellConstraints.DEFAULT, CellConstraints.CENTER));
        btnPlayCompareFile = new JButton();
        btnPlayCompareFile.setText("play");
        panel1.add(btnPlayCompareFile, cc.xy(5, 3));
        midiPanel = new JPanel();
        midiPanel.setLayout(new BorderLayout(0, 0));
        midiPanel.setBackground(new Color(-1));
        midiPanel.setMinimumSize(new Dimension(100, 100));
        midiPanel.setPreferredSize(new Dimension(10, 200));
        panel1.add(midiPanel, cc.xywh(1, 11, 5, 3, CellConstraints.DEFAULT, CellConstraints.FILL));
        midiPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(new Color(-16777216)), null));
        final JScrollPane scrollPane2 = new JScrollPane();
        scrollPane2.setHorizontalScrollBarPolicy(31);
        scrollPane2.setVerticalScrollBarPolicy(20);
        panel1.add(scrollPane2, cc.xywh(7, 5, 3, 5, CellConstraints.FILL, CellConstraints.FILL));
        featuresContainerPanel = new JPanel();
        featuresContainerPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
        featuresContainerPanel.setAlignmentX(0.0f);
        featuresContainerPanel.setBackground(new Color(-986896));
        featuresContainerPanel.setMinimumSize(new Dimension(300, 100));
        featuresContainerPanel.setPreferredSize(new Dimension(300, 100));
        scrollPane2.setViewportView(featuresContainerPanel);
        btnPlayTableSelection = new JButton();
        btnPlayTableSelection.setText("play table selection");
        panel1.add(btnPlayTableSelection, cc.xy(7, 13));
    }

    /**
     * @noinspection ALL
     */
    public JComponent $$$getRootComponent$$$() {
        return panel1;
    }

    private class MIDIFileFilter extends FileFilter {
        public boolean accept(File f) {
            return f.isDirectory() || f.getName().toLowerCase().endsWith(".mid");
        }

        public String getDescription() {
            return ".mid files";
        }
    }

    private void printTA(String txt) {
        taOutput.append(txt + "\n");
    }

    private void clearTA() {
        taOutput.setText("");
    }

    private void setMidiChkBoxes(String[] files) {
        midiPanel.removeAll();
        JCheckBox ckb;
        JButton btnPlay;

        FormLayout layout = new FormLayout("25dlu, 4dlu, left:max(40dlu;p)", "");
        DefaultFormBuilder builder = new DefaultFormBuilder(layout);

        int s = 30;
        for (final String file : files) {

            //printTA(file);
            ckb = new JCheckBox(file.replace(".mid", "")); //substring(0, file.indexOf(".mid")));
            btnPlay = new JButton("p");
            builder.append(btnPlay);
            builder.append(ckb);

            final JCheckBox ckb1 = ckb;
            ckb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (ckb1.isSelected()) {
                        compFiles.add(file);
                    } else {
                        compFiles.remove(file);
                    }
                }
            });
            //s = ckb.getHeight();
            btnPlay.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    for (ACMeta m : comp.lib.mfiles) {
                        if (m.relPath.equals(file)) {
                            File f = new File(m.sourceFile);
                            if (f.exists()) {
                                printTA("playing " + m.sourceFile);

                                Score ps = new Score();
                                Read.midi(ps, m.sourceFile);
                                Play.midi(ps, false);

                            } else {
                                printTA("could not find source file " + m.sourceFile);
                            }
                        }
                    }
                }
            });
        }

        JPanel zPanel = builder.getPanel();
        ScrollPane sp = new ScrollPane();
        sp.add(zPanel);
        midiPanel.add(sp);
        //midiPanel.setSize(midiPanel.getWidth(), midiPanel.getComponentCount() * s);
        midiPanel.validate();

        //midiScrollPane.add(zPanel);
        //midiScrollPane.repaint();

    }

    private void createOutputTable(Hashtable<ACMeta, Hashtable<String, Double>> ht) {
        tablePanel.removeAll();

        String[] columnNames = new String[features.length + 2];
        columnNames[0] = "File";
        columnNames[1] = "Total";
        for (int i = 2; i < features.length + 2; i++) {
            columnNames[i] = features[i - 2];
        }

        Object[][] data = new Object[ht.keySet().size()][features.length + 2];

        int i = 0;

        for (ACMeta ag : ht.keySet()) {
            data[i][0] = ag.relPath;//ag.sourceFile.substring(ag.sourceFile.lastIndexOf('/') + 1, ag.sourceFile.length());
            data[i][1] = ht.get(ag).get("[total]");

            for (String f : ht.get(ag).keySet()) {
                if (f.equals("[total]")) continue;
                int oPos = Integer.parseInt(f.substring(0, f.indexOf(':')));
                data[i][oPos + 2] = ht.get(ag).get(f);
            }
            i++;
        }

        TableModel model = new DefaultTableModel(data, columnNames);
        outputTable = new JTable(model);

        JScrollPane spOutputTable = new JScrollPane(outputTable);
        outputTable.setPreferredScrollableViewportSize(new Dimension(650, 160));
        outputTable.setAutoCreateRowSorter(true);

        tablePanel.add(spOutputTable);
        tablePanel.repaint();
        tablePanel.validate();

    }

}
