package gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

/**
 * image display and manipulation control
 *
 * @author  Lukas Maczejka, Viktor Pavlu
 * @version 1.0
 */
public class MoveControl extends JPanel implements MouseListener, MouseMotionListener, ActionListener
{
  public boolean movable = false;
  public boolean showSaveButton  = false;
  public boolean showZoomButton  = false;
  public boolean showCloseButton = false;

  public static int SIZE_X = 200;
  public static int SIZE_Y = 200;

  public Color titleBackColor  = new Color(80, 80, 190);
  public Color backColor       = Color.lightGray;
  public Color titleTextColor  = Color.white;
  public Color titleHoverColor = Color.blue;

  private int mposx, mposy;

  private Image offScreenBuffer;

  public JLabel contentPanel;

  public  JPanel controlPanel;
  public  JPanel titlePanel;
  private JLabel titleLabel;

  public String title;

  private JLabel closeButton;
  private JLabel saveButton;
  private JLabel zoomButton;

  private JPanel buttonPanel;

  /**
   * sets initial control appearance
   */
  public void initializeComponents()
  {
    setLayout(new BorderLayout());
    setBorder(BorderFactory.createLineBorder(Color.black));
    setSize(SIZE_X,SIZE_Y);
    setPreferredSize(new Dimension(SIZE_X,SIZE_Y));
    //setBackground(backColor);

    titlePanel = new JPanel();
    titlePanel.setLayout(new BorderLayout());
    titlePanel.setBackground(titleBackColor);
    titlePanel.setBorder(BorderFactory.createEtchedBorder(Color.white, Color.black));

    titleLabel = new JLabel(" " + title);
    titleLabel.setForeground(titleTextColor);
    titleLabel.setHorizontalTextPosition(JLabel.RIGHT);

    buttonPanel = new JPanel(new BorderLayout());

    if (showCloseButton)
    {
      //BufferedImage closeImage = CVImage.loadImageFromFile("./close.png");
      closeButton = new JLabel("");
      closeButton.setBorder(BorderFactory.createBevelBorder(0));
      buttonPanel.add(closeButton, BorderLayout.EAST);
    }
    if (showZoomButton)
    {
      //BufferedImage zoomImage = CVImage.loadImageFromFile("./zoom.png");
      zoomButton = new JLabel("");
      zoomButton.setBorder(BorderFactory.createBevelBorder(0));
      buttonPanel.add(zoomButton, BorderLayout.CENTER);
    }

    if (showSaveButton)
    {
      //BufferedImage saveImage = CVImage.loadImageFromFile("./save.png");
      saveButton = new JLabel("");
      saveButton.setBorder(BorderFactory.createBevelBorder(0));
      buttonPanel.add(saveButton, BorderLayout.WEST);
    }

    titlePanel.add(buttonPanel, BorderLayout.EAST);

    titlePanel.add(titleLabel, BorderLayout.CENTER);

    controlPanel = new JPanel();
    controlPanel.setLayout(new BorderLayout());

    JPanel center = new JPanel();
    center.setLayout(new BorderLayout());

    contentPanel = new JLabel();
    contentPanel.setPreferredSize(new Dimension(SIZE_X,SIZE_Y));
    contentPanel.setBorder(BorderFactory.createEtchedBorder(Color.white, Color.black));
    contentPanel.setHorizontalAlignment(SwingConstants.CENTER);

    add(titlePanel, BorderLayout.NORTH);
    add(contentPanel, BorderLayout.CENTER);
    add(controlPanel, BorderLayout.SOUTH);
  }

  public void setTitle(String title)
  {
    titleLabel.setText(title);
  }

  /**
   * constuctor
   * initialises image control
   *
   */
  public MoveControl(String title)
  {
    this.title = title;

    addMouseListener(this);
    addMouseMotionListener(this);

    initializeComponents();
  }

  public void actionPerformed( ActionEvent evt )
	{

		if (evt.getActionCommand().equals("close"))
		{
		  getParent().remove(this);
		}

	}


	/**
   * double buffering
   *
   * @param g Graphics object
   */
  public void update(Graphics g)
  {
    Graphics gr;
    if (offScreenBuffer == null || (! (offScreenBuffer.getWidth(this) == getSize().width && offScreenBuffer.getHeight(this) == getSize().height)))
    {
      offScreenBuffer = createImage(getSize().width, getSize().height);
    }
    gr = offScreenBuffer.getGraphics();

    paint(gr);

    g.drawImage(offScreenBuffer, 0, 0, this);
  }


  /**
   * MouseMove event handler
   *
   * @param e MouseEvent
   */
  public void mouseMoved(MouseEvent e)
  {
    mposx = e.getX();
    mposy = e.getY();
    repaint();
  }

  /**
   * MouseDragged event handler
   * moves the control
   *
   * @param e MouseEvent
   */
  public void mouseDragged(MouseEvent e)
  {
    int mousex = e.getX(),
        mousey = e.getY();

    if (getComponentAt(mposx, mposy) == titlePanel && titlePanel.getComponentAt(mposx, mposy) == titleLabel && movable)
      setLocation(getLocation().x + mousex-mposx, getLocation().y + mousey-mposy);
  }

  /**
   * MousePressed event handler
   *
   * @param e MouseEvent
   */
  public void mousePressed(MouseEvent e)
  {
    if (getComponentAt(mposx, mposy) == titlePanel && titlePanel.getComponentAt(mposx, mposy) == buttonPanel)
    {
      JLabel clicked = (JLabel)buttonPanel.getComponentAt((mposx - buttonPanel.getLocation().x), (mposy - buttonPanel.getLocation().y));
      clicked.setBorder(BorderFactory.createBevelBorder(1));
    }
  }

  /**
   * MouseReleased event handler
   *
   * @param e MouseEvent
   */
  public void mouseReleased(MouseEvent e)
  {
    if (getComponentAt(mposx, mposy) == titlePanel && titlePanel.getComponentAt(mposx, mposy) == buttonPanel)
    {
      JLabel clicked = (JLabel)buttonPanel.getComponentAt((mposx - buttonPanel.getLocation().x), (mposy - buttonPanel.getLocation().y));
      clicked.setBorder(BorderFactory.createBevelBorder(0));
    }
  }

  /**
   * MouseExited event handler
   * set title color to inactive color
   *
   * @param e MouseEvent
   */
  public void mouseExited(MouseEvent e)
  {
    titlePanel.setBackground(titleBackColor);
  }

  /**
   * MouseEntered event handler
   * set title color to active color
   *
   * @param e MouseEvent
   */
  public void mouseEntered(MouseEvent e)
  {
    titlePanel.setBackground(titleHoverColor);
  }

   /**
   * MouseClicked event handler
   * open detailed image view
   *
   * @param e MouseEvent
   */
  public void mouseClicked(MouseEvent e)
  {
    if (getComponentAt(mposx, mposy) == contentPanel)
    {
    }
  }

  /**
   * Computes ImageCotrols' function
   *
   */
  public void Compute()
  {
  }
}
