package main;
/***

main class, console ui

2009-05-20	mac		created

 **/
 
import features.ACFeature;

import java.util.*;
import java.io.*;

public class ACompare
{
	public ACLibrary lib;
	
	public ACompare()
	{
		lib = new ACLibrary();
	}
	
	public static void main (String[] args)
	{
		ACompare comp = new ACompare();
		
	  if (args.length == 2 && args[0].equals("-lib"))
	  {
 			comp.lib.loadMeta(args[1]);
		}
		
		else if (args.length == 3 && args[0].equals("-print"))
		{
		   comp.lib.loadMeta(args[1]);
		   comp.lib.print(args[2]);
		}
		
		else if (args.length == 3 && args[0].equals("-load"))
		{
		  comp.lib.analyseMeta(args[1], args[2]);

		}

      else if (args.length == 3 && args[0].equals("-compare"))
      {
          comp.lib.loadMeta(args[1]);

          int i = 0;

          /*
          System.out.println("Parsons Code");
          System.out.println("------");

          for (Double d : comp.compareAgainstLibrary(args[2], "Parsons Code"))
          {
            System.out.print(comp.lib.mfiles.get(i++).sourceFile);
            System.out.println("=" + d);
          }        */
         /*
          i = 0;
          System.out.println("Note Duration");
          System.out.println("------");

          for (Double d : comp.compareAgainstLibrary(args[2], "Note Duration"))
          {
            System.out.print(comp.lib.mfiles.get(i++).sourceFile);
            System.out.println("=" + d);
          }  */

          ACMeta in = new ACMeta();
          in.analyseFile(args[2]);

          String[] feats = ACFeature.getFeatureIds();


          String[] comps = new String[6];
          comps[0] = "Euclidean Distance";
          comps[1] = "Hamming Distance";
          comps[2] = "Hamming Distance";
          comps[3] = "Euclidean Distance";
          comps[4] = "Levenshtein Distance";
          comps[5] = "";

          double[] weights = new double[6];
          weights[0] = 1d;
          weights[1] = 1d;
          weights[2] = 1d;
          weights[3] = 1d;
          weights[4] = 1d;
          weights[5] = 0d;

          System.out.println(printResults(compareAllAgainstList(in, comp.lib.mfiles, feats, comps, weights)));
      }
		
			
		else
		{
		  System.out.println("\nusage: java ACompare\t ");
		  System.out.println("\t -lib <libdir>                 load library folder");
		  System.out.println("\t -load <mididir> <libdir>      analyse midi folder, store to library");
		  System.out.println("\t -print <libdir> <filename>    library file information");
		  System.out.println("\t -compare <libdir> <midifile>  compare midi file with library");
		}
		
		comp.status();
	}
	
	public void status()
	{
		System.out.println("");
		System.out.println("status");
		System.out.println("---------");
		
		lib.status();
	}


    public ArrayList<Double> compareAgainstLibrary(String midiin, String featureID)
    {
        return compareAgainstLibrary(midiin, lib.mfiles, featureID);
    }
	
	public static ArrayList<Double> compareAgainstLibrary(String midiin, ArrayList<ACMeta> metas, String featureID)
    {
        ArrayList<Double> results = new ArrayList<Double>();
        //load midiin

        ACMeta in = new ACMeta();
        in.analyseFile(midiin);
        ACFeature fin = in.getFeatureByID(featureID);
        if (fin == null) return results;

        for (ACMeta ml : metas)
        {
            ACFeature f = ml.getFeatureByID(featureID);

            if (f != null)
                results.add(f.compare(fin));        
        }

        return results;
    }


    public static ArrayList<Double> compareAgainstList(ACMeta cagainst, ArrayList<ACMeta> metas, String featureID, String compID)
    {
        System.out.println("compare against list");
        ArrayList<Double> results = new ArrayList<Double>();

        ACFeature fin = cagainst.getFeatureByID(featureID);
        if (fin == null) return results;

        System.out.println("compare " + featureID);
        
       for (ACMeta ml : metas)
        {

            ACFeature f = ml.getFeatureByID(featureID);
            f.compareAlgorithm = compID;

            if (f != null)
                results.add(f.compare(fin));
        }

        return results;
    }

    public static Hashtable<ACMeta,Hashtable<String,Double>> compareAllAgainstList(ACMeta cagainst, ArrayList<ACMeta> metas, String[] featureID, String[] compID, double[] weights)
    {
        System.out.println("compare all against list");
        Hashtable<ACMeta,Hashtable<String,Double>> results = new  Hashtable<ACMeta,Hashtable<String,Double>>();

        for (ACMeta ml : metas)
        {

            double tot = 0d;
            double count = 0d;

            Hashtable<String,Double> line = new Hashtable<String,Double>();

            for (int i = 0; i < featureID.length; i++)
            {
                System.out.println("compare feature [" + featureID[i] + "] against " + ml.sourceFile);
               ACFeature f = ml.getFeatureByID(featureID[i]);

               ACFeature fin = cagainst.getFeatureByID(featureID[i]);
                if (f == null) System.out.println("compare feature not found");
               if (fin == null || f == null) continue;

               fin.compareAlgorithm = compID[i];

               double val = fin.compare(f);
               System.out.println("put " + featureID[i] + "/" + compID[i] + " (" + val + ")");
               line.put(i + ":" + featureID[i] + "/" + compID[i], val);

               tot += val*weights[i];
               count += weights[i];

            }
            line.put("[total]", (tot/count));

            results.put(ml, line);
        }

        return results;
    }

    public static String printResults(Hashtable<ACMeta,Hashtable<String,Double>> ht)
    {
        String res = "";

        for (ACMeta ag : ht.keySet())
        {
            res += ag.sourceFile + "\n";

            res +=  "[total]\t" + ht.get(ag).get("[total]") + "\n";

            for (String f : ht.get(ag).keySet())
            {
                if (f.equals("[total]")) continue;
                res += f + "\t" + ht.get(ag).get(f) + "\n";
            }

            res += "\n\n";
        }

        return res;
    }

}